import os
import subprocess
import sys
from pathlib import Path

# 颜色常量
COLOR_RED = "\033[31m"
COLOR_GREEN = "\033[32m"
COLOR_YELLOW = "\033[33m"
COLOR_BLUE = "\033[34m"
COLOR_RESET = "\033[0m"

def generate_nginx_config(port, static_root, server_name):
    """生成与模板完全一致的nginx配置"""
    static_root = static_root.rstrip('/') + '/'
    
    return f"""
user nginx;
worker_processes auto;
error_log /var/log/nginx/error.log;
pid /run/nginx.pid;

# Load dynamic modules. See /usr/share/doc/nginx/README.dynamic.
include /usr/share/nginx/modules/*.conf;

events {{
    worker_connections 1024;
}}

http {{
    log_format  main  '$remote_addr - $remote_user [$time_local] "$request" '
                      '$status $body_bytes_sent "$http_referer" '
                      '"$http_user_agent" "$http_x_forwarded_for"';

    access_log  /var/log/nginx/access.log  main;

    sendfile            on;
    tcp_nopush          on;
    tcp_nodelay         on;
    keepalive_timeout   65;
    types_hash_max_size 4096;

    include             /etc/nginx/mime.types;
    default_type        application/octet-stream;

    # Load modular configuration files from the /etc/nginx/conf.d directory.
    # See http://nginx.org/en/docs/ngx_core_module.html#include
    # for more information.
    # include /etc/nginx/conf.d/*.conf;

    upstream django {{
        server 127.0.0.1:{port};
    }}

    server {{
        listen       80;
        listen       [::]:80;
        server_name  {server_name};

        # Load configuration files for the default server block.
        # include /etc/nginx/default.d/*.conf;

        location /static/ {{
            alias   {static_root};
        }}

        location / {{
            uwsgi_pass  django;
            include     uwsgi_params;
        }}
    }}
}}
"""

def configure_nginx(port, static_root, server_name):
    """配置nginx主配置文件"""
    nginx_conf_path = "/etc/nginx/nginx.conf"
    
    # 备份原配置
    if os.path.exists(nginx_conf_path):
        backup_path = "/etc/nginx/nginx.conf.bak"
        subprocess.run(f"cp {nginx_conf_path} {backup_path}", shell=True)
        print(f"{COLOR_GREEN}✔ 原配置已备份到 {backup_path}{COLOR_RESET}")
    
    # 生成并写入新配置
    config = generate_nginx_config(port, static_root, server_name)
    try:
        with open(nginx_conf_path, 'w') as f:
            f.write(config.strip())
        print(f"{COLOR_GREEN}✔ 新配置已写入 {nginx_conf_path}{COLOR_RESET}")
        
        # 测试配置
        test_result = subprocess.run(
            "nginx -t",
            shell=True,
            capture_output=True,
            text=True
        )
        if test_result.returncode != 0:
            print(f"{COLOR_RED}Nginx配置测试失败:{COLOR_RESET}")
            print(test_result.stderr)
            sys.exit(1)
        print(f"{COLOR_GREEN}✔ Nginx配置测试通过{COLOR_RESET}")
        
    except Exception as e:
        print(f"{COLOR_RED}错误: 写入配置失败 - {str(e)}{COLOR_RESET}")
        sys.exit(1)

def main():
    # 检查root权限
    if os.getuid() != 0:
        print(f"{COLOR_RED}错误: 请使用root用户运行此脚本{COLOR_RESET}")
        sys.exit(1)

    print(f"{COLOR_BLUE}=== Django项目Nginx部署工具 ==={COLOR_RESET}")

    # 1. 获取项目配置
    project_root = os.getcwd()
    while not os.path.exists(os.path.join(project_root, 'manage.py')):
        project_root = os.path.dirname(project_root)
        if project_root == '/':
            print(f"{COLOR_RED}错误: 未找到manage.py{COLOR_RESET}")
            sys.exit(1)
    print(f"{COLOR_GREEN}✔ 项目根目录: {project_root}{COLOR_RESET}")

    # 2. 获取静态文件路径
    static_root = os.path.abspath(os.path.join(project_root, "../SemiAutomaticChargeSystem_ALL_STATICFILES"))
    if not os.path.exists(static_root):
        os.makedirs(static_root, exist_ok=True)
        print(f"{COLOR_YELLOW}警告: 静态文件目录不存在，已创建: {static_root}{COLOR_RESET}")
    print(f"{COLOR_GREEN}✔ 静态文件目录: {static_root}{COLOR_RESET}")

    # 3. 获取用户输入
    port = input(f"{COLOR_BLUE}请输入Django运行端口(默认8001): {COLOR_RESET}") or "8001"
    server_name = input(f"{COLOR_BLUE}请输入server_name(默认day06.pythonav.com): {COLOR_RESET}") or "day06.pythonav.com"

    # 4. 安装Nginx
    print(f"{COLOR_BLUE}正在安装Nginx...{COLOR_RESET}")
    subprocess.run("yum install -y nginx", shell=True, check=True)
    subprocess.run("yum install -y uwsgi uwsgi-plugin-python3", shell=True, check=True)

    # 5. 配置并启动
    configure_nginx(port, static_root, server_name)
    subprocess.run("systemctl restart nginx", shell=True, check=True)
    subprocess.run("systemctl enable nginx", shell=True, check=True)

    # 6. 输出结果
    print(f"\n{COLOR_GREEN}部署成功!{COLOR_RESET}")
    print(f"静态文件目录: {static_root}")
    print(f"Django运行端口: {port}")
    print(f"Server Name: {server_name}")
    print(f"\n后续步骤:")
    print(f"1. 收集静态文件: python manage.py collectstatic --noinput")
    print(f"2. 启动uWSGI: uwsgi --ini your_uwsgi.ini")

if __name__ == "__main__":
    main()